﻿using BA.API;
using BA.Core.Configuration.Navigation;
using BA.Core.Configuration.Navigation.Client;
using BA.Core.Configuration.OrmEntity;
using BA.Core.CustomAttributes;
using BA.Core.DataProviders;
using BA.Core.DB;
using BA.Core.Enums;
using BA.Core.Models;
using $safeprojectname$.Enums.Extensions;
using $safeprojectname$.Igniters;
using DevExpress.Web;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Linq.Expressions;

namespace $safeprojectname$.Configuration.Navigation
{
    /// <summary>
    /// Aktion mit einem Igniter. Die Funktionalität wird synchron ausgeführt und der Benutzer muss warten.
    /// </summary>
    [Serializable]
    [Toolbox(EnumConfigurationType.NavigationConfigurationGuid, true)]
    [ControlFilter(nameof(NavigationConfiguration.NavigationConfigurationType), ExpressionType.Equal, EnumNavigationConfigurationType.RibbonNavigationGuid, EnumControlFilterApplyState.OptIn)]
    public class ClientActionAddService : ClientActionGridMassOperationBase
    {
        [DisplayName("41ABA083-E37A-4709-98D4-1D685496459C")]
        public string RemarkMessage { get; set; }

        [TokenboxControl]
        [CDPRolesProviderProperties(dataSources: new[] { EnumDataSource.RoleGuid })]
        [DisplayName("BD0D11A9-9E71-4980-9265-C4A037432D48")]
        public RoleSet DialogRoles { get; set; }

        public ClientActionAddService() : base()
        {
            ToolboxName = "DEBED3B0-36A7-4948-B647-116D76A64504";
            Caption = "DEBED3B0-36A7-4948-B647-116D76A64504";
            ControlInitName = "TrainingActionAddService";
            ToolboxGroupName = "C007681C-8644-4BB0-A4A0-4A643265EABD";
            Id = "97C171CA-F872-4245-8AC5-43BF169155E7".ToGuid();
            Icon = "wrench";
            IconName = Icon;

            // Action ist nur für Ansichten
            VisibilityForParentTypes.Add(EnumActionVisibleForParentType.Grid);

            // Igniter setzen
            MassOperationIgniter = typeof(AddServiceIgniter).AssemblyQualifiedName;

            // Sichtbarkeitssteuerung bei welcher von Selektionen die Aktion sichtbar ist
            // Weitere Möglichkeiten: OneSelected, OneOrNothingSelectedGuid, NothingSelected, ManySelected
            DynamicClientVisibility.Add(EnumActionVisibility.SomethingSelected);

            // Wird in CanHandleOrm mit tru oder false belegt
            DynamicClientVisibility.Add(EnumActionVisibility.IfUserHasRole);

            // Clientseitige Aktion, die ausgeführt wird
            ActionMethodId = "$safeprojectname$.ClientActionAddService";
        }

        /// <inheritdoc/>
        public override void AdditionalRibbonButtonAssignment(RibbonButtonItem ribbonItem, Dictionary<string, object> additionalClientData, EnumActionVisibleForParentType parentType, DevExUIModelBase uiModel, OrmBABase orm)
        {
            // Die Basemethode sollte immer als erste ausgeführt werden
            base.AdditionalRibbonButtonAssignment(ribbonItem, additionalClientData, parentType, uiModel, orm);

            // Übertragung der konfigurierten Werte zum Client
            additionalClientData.AddOrUpdate("RemarkMessage", RemarkMessage);
            additionalClientData.AddOrUpdate("DialogAuthorized", Api.User.CurrentUserIsInRole(DialogRoles, false));

            // Ermittlung ob die Aktion ausgeführt werden kann
            bool canHandle;
            if (orm != null)
                // Überprüfung eines konkreten Orms, ob der Benutzer es bearbeiten kann. Für Aktionen in Masken.
                // Hinweis: Oben wurde die Aktion nur für Ansichten definiert. Daher wird dieser Abschnitt
                // nicht durchlaufen und dient nur als Beispiel
                canHandle = orm.IsAllowed(EnumTableOperations.Edit);
            else
            {
                // In einer Ansicht kann nicht ein konkretes Orm angefragt werden, daher wird nur geprüft ob der Benutzer theoritisch
                // Datensätze dieses Typs bearbeiten kann. Dazu wird die Datentabellenkonfiguration geladen.
                OrmEntityConfiguration entityConfig = Api.Config.OrmEntity(EnumDataSourceExtension.Engine.ValueGuid);
                canHandle = entityConfig.IsAllowed(EnumTableOperations.Edit) != EnumTableOperations.Denied;
            }

            // Ergebnis zum Client übertragen. Siehe dazu im Konstruktor DynamicClientVisibility.Add(EnumActionVisibility.IfUserHasRole);
            additionalClientData.AddOrUpdate("UserHasRole", canHandle);
        }
    }
}