﻿using BA.API;
using BA.API.Relations;
using BA.Core.CriteriaOperators;
using BA.Core.DataProviders;
using BA.Core.DB;
using BA.Core.Models;
using $safeprojectname$.DB;
using $safeprojectname$.Enums.Extensions;
using DevExpress.Data.Filtering;
using DevExpress.Xpo;
using System;
using System.Linq;

namespace $safeprojectname$.DataProvider
{
    /// <summary>
    /// Eigener Ansichten Datenprovider für die Maschinen
    /// </summary>
    public class GDPEngines : IGridDataProvider
    {
        /// <inheritdoc/>
        public GridDataProviderPropertiesBase Properties { get; set; }

        /// <inheritdoc/>
        public IQueryable<object> GetQuery(GridDataProviderParameter parameter = null)
        {
            // Ein Griddatenprovider liefert ein IQueryable. Dies muss nicht ein DB-Query sein, wie in diesem Beispiel. Wo die Daten
            // herkommen spielt keine Rolle.
            // In diesem Beispiel wird ein DB-Query erstellt. Die hinzugefügte Bedingung (Where) ist einmal in C# und einmal als Formula
            // implementiert. 

            // Hinweis: In der Regel werden die Griddatenprovider in C# implementiert. Die Formelbeispiele dienen an dieser Stelle nur als 
            // weitere Beispiele für den Umgang mit diesen.

            if (Properties is GDPEnginesProperties myProperties)
            {
                // Datenbank Query erstellen
                Session session = Api.ORM.GetNewSession();
                IQueryable<OrmEngine> enginesQuery;
                if (myProperties.WithReadPermissions)
                    enginesQuery = Api.ORM.GetQueryWithReadPermissions<OrmEngine>(session);
                else
                    enginesQuery = Api.ORM.GetQuery<OrmEngine>(session);

                // Filter hinzufügen. Einmal  direkt in C# und einmal als Formel
                if (!string.IsNullOrEmpty(myProperties.ResponsibleNotContains))
                    // C#: Alle Maschinen die im Betreuerfeld den Propertywert nicht haben.
                    enginesQuery = enginesQuery.Where(ff => !ff.Responsible.Contains(myProperties.ResponsibleNotContains));
                else
                {
                    // Formel: Alle Maschinen deren Betreuerfeld nicht NULL sind
                    CriteriaOperator criteriaWhere = new OperandProperty(nameof(OrmEngine.Responsible)) != new OperandValue(null);
                    enginesQuery = enginesQuery.Where(ff => ff.Fit(criteriaWhere));
                }

                return enginesQuery;
            }

            throw new Exception("Wrong properties");
        }

        /// <inheritdoc/>
        public IQueryable<GridDataProviderRowBase> GetData(GridDataProviderParameter parameter = null)
        {
            // Um die Abfrage in einer Ansicht anzuzeigen, müssen noch die jeweiligen Spaltenwerte generiert werden.Die Spalte
            // "Betreuer aus Relation" ist einmal in C# und einmal als Formel.

            // Hinweis: In der Regel werden die Griddatenprovider in C# implementiert. Die Formelbeispiele dienen an dieser Stelle nur als 
            // weitere Beispiele für den Umgang mit diesen.

            if (Properties is GDPEnginesProperties myProperties)
            {
                // Datenbank Query erstellen
                IQueryable<OrmEngine> enginesQuery = (IQueryable<OrmEngine>)GetQuery(parameter);
                Session session = ((XPQuery<OrmEngine>)enginesQuery).Session;

                // Formel: Betreuer Relation
                CriteriaOperator expression = !CriteriaOperatorBuilder.GetSourcesJoin(true, EnumRelationTypeExtension.Responsible, null, onlyRelations: true, aggregator: Aggregate.Exists);
                CriteriaOperator ifpart = new OperandValue("Keine Betreuer Relation");
                CriteriaOperator elsepart = CriteriaOperatorBuilder.GetSourcesJoin(true, EnumRelationTypeExtension.Responsible, null, aggregatedExpression: new OperandProperty(nameof(OrmEngine.EntityTitle)));
                CriteriaOperator coResponsibleRel = new FunctionOperator(FunctionOperatorType.Iif, expression, ifpart, elsepart);

                // C#: Betreuer Relation
                // Abfrage auf die Relationstabelle
                IQueryable<OrmRelation> relationQuery = session.Query<OrmRelation>().Where(ff => ff.RelationType == EnumRelationTypeExtension.Responsible.ValueGuid);
                // Die primären Betreuerrelationen
                relationQuery = relationQuery.Where(ff => ff.RelationType == EnumRelationTypeExtension.Responsible.ValueGuid && ff.IsPrimary);
                // Join mit den Maschinen. Die Maschine ist das Ziel (Target)
                var query = enginesQuery.Join(relationQuery, e => e.Oid, r => r.Target, (e, r) => new { e, r });
                // Abfrage auf die Benutzerprofile
                IQueryable<OrmUserProfile> userProfilesQuery = Api.ORM.GetQuery<OrmUserProfile>(session);
                // Join mit den Machinen und Betreuerrelationen. Benutzerprofile sind die Quelle (Source).
                // Ergebnis sind die Maschinen mit den entsprechenden Betreuer
                var resultQuery = query.Join(userProfilesQuery, q => q.r.Source, u => u.Oid, (q, u) => new { engine = q.e, responsible = u });

                IQueryable<GridResponsibleRowModel> selectQuery = resultQuery.Select(ff => new GridResponsibleRowModel
                {
                    Oid = ff.engine.Oid,
                    RowId = ff.engine.Oid,
                    Name = ff.engine.Name,
                    ResponsibleText = ff.engine.Responsible,
                    ResponsibleRelationFormula = (string)ff.engine.Evaluate(coResponsibleRel),
                    ResponsibleRelationLINQ = ff.responsible.EntityTitle,
                });

                return selectQuery;
            }

            throw new Exception("Wrong properties");
        }
    }
}