declare module BA.Ui.TabController {
    class Controller {
        static debug: any;
        static link_AppBase_Url: string;
        static link_Base_Url: string;
        static link_UiTabControl_SetTabs: string;
        static link_Start_Url: string;
        static link_CheckAlive_Url: string;
        static isMobile: boolean;
        rightClickedTab: string;
        tabForPopUpMenuActions: string;
        TabsRemoved: boolean;
        private static _Instance;
        private _ActiveTab;
        private _Tabs;
        private _TabToActivateAfterLoad;
        private _TabToCloseAfterLoad;
        private _IFrameAreaName;
        private _TabsAreaName;
        private _UpdateHistory;
        private _WaitAnimationVisible;
        private _WaitAnimationCounter;
        private _WaitAnimationTimeout;
        private _PoolInterWindowTimerHandle;
        private _CheckAliveTimerHandle;
        private _OnNextTabLoaded;
        private _ServerIsOffline;
        private _ApplicationRunningInstanceId;
        private _State;
        private _LastError;
        static GetInstance(): Controller;
        constructor();
        InitDragAndDrop(): void;
        private ActivateTab;
        UpdateTabBarIfTabsRemoved(): void;
        UpdateTabBar(renderTabs?: boolean): void;
        get UpdateHistory(): boolean;
        set UpdateHistory(value: boolean);
        get ActiveTab(): Tab;
        /**
         * Setter for the active tab property. It handles also the delayed loading / activation and the history update.
         */
        set ActiveTab(value: Tab);
        /**
         * Setter for the property containing the method to be executed next time that a tab is loaded
         */
        get OnNextTabLoaded(): Function;
        /**
         * Getter for the property containing the method to be executed next time that a tab is loaded
         */
        set OnNextTabLoaded(value: Function);
        /**
         * Array of Tabs (readonly)
         */
        get Tabs(): Array<Tab>;
        /**
         * Setter of the ID of the <div> for the IFrames
         */
        set IFrameAreaName(value: string);
        /**
         * Getter of the ID of the <div> for the IFrames
         */
        get IFrameAreaName(): string;
        /**
         * Setter of the ID of the <div> for the tab headers
         */
        set TabsAreaName(value: string);
        /**
         * Getter of the ID of the <div> for the tab headers
         */
        get TabsAreaName(): string;
        /**
         * Getter of the property for the tab that should be closed automatically after activating a new one
         */
        get CloseTabAfterLoad(): Tab;
        /**
         * Setter of the property for the tab that should be closed automatically after activating a new one
         */
        set CloseTabAfterLoad(tab: Tab);
        /**
         * Setter of the flag that indicates if a load animation is shown to the user bz this controller
         */
        set WaitAnimationVisible(value: boolean);
        /**
         * Getter of the flag that indicates if a load animation is shown to the user bz this controller
         */
        get WaitAnimationVisible(): boolean;
        /**
         * The current state of the controller
         */
        get State(): ControllerState;
        /**
         * Setter of the property for he last error of the controller or the toast
         */
        set LastError(value: string);
        /**
         * Getter of the property for he last error of the controller or the toast
         */
        get LastError(): string;
        /**
         * Update Content of tab
         *
         * @param tab: tab to be updated
         * @param url: new url to load
         * @param overrideTabReference: the tab reference to be used. Default empty string
         * @param replaceHistory: if the history entry in browser should be replace. Default false
         */
        UpdateTabContent(tab: Tab, url: string, overrideTabReference?: string, replaceHistory?: boolean): void;
        /**
             * Creates a refernce for the tab
             *
             * @param url: url to calculate the reference
             */
        CreateTabReference(url: string): string;
        /**
         * Create a tab. This method will create a new tab if no tab already exists with the same reference if the tab should be unique.
         *
         * @param url: url of the tab to open
         * @param title: title for the tab. Default no title
         * @param icon: icon for the tab. Default no icon
         * @param delayLoading: of its loading should be delayed. Default false.
         * @param guid: identifier for the tab. Default null
         * @param andActivate: if the tab should be activated after being added or recovered. Default true
         * @param initialLoad: if the method is being called during an initialization of the tab system. Default false
         * @param overrideTabReference: if the tab reference should be a specific, given string instead of the url of the tab. Default empty string
         * @param asLastVisibleTab: if the tab should be added at the end of the tab bar. Default false
         * @param isUniqueTab: if the tab should be treated as unique. Default true
         * @param predecessor: if specified the tab will be created next to this tab, otherwise it will be opened next to active tab. Default null
         */
        AddTab(url: string, title?: string, icon?: string, delayLoading?: boolean, guid?: BA.Classes.Guid, andActivate?: boolean, initialLoad?: boolean, overrideTabReference?: string, asLastVisibleTab?: boolean, isUniqueTab?: boolean, isPinned?: boolean, predecessor?: Tab): Tab;
        /**
         * Removes a tab from the system if the tab is not dirty, or if even if it is dirty, the user confirms they want to close it.
         * If it was the last tab, a start page will be opened
         *
         * @param tab: Tab to remove
         */
        RemoveTab(tab: Tab, autoUpdateTabBar?: boolean): void;
        /**
         * Perform the different clean up tasks necessary when a tab is closed
         *
         * @param tab: tab closed
         * @param hiddenData: its hidden data
         */
        static CleanUpAtClose(tab: Tab, hiddenData: BA.Ui.Models.FormHiddenDataModel): void;
        private _RemoveTab;
        /**
         * Set the focus to the current tab content
         */
        FocusActiveTab(): void;
        /**
         * Checks if any tab in the list has been marked as dirty
         */
        IsAnyTabDirty(): boolean;
        /**
         * Sets all the tab as clear (not dirty)
         */
        ClearAllTabs(): void;
        /**
         * Close all possible tabs (!isDirty), but skip one and the pinned, if set so in parameters
         *
         * @param tabToSkip: tab to skip. Default null. It can be a tab, or the identifier of the desired tab
         * @param skipPinned: flag indicating if the pinned tabs should also be skipped (not removed)
         */
        CloseAllTabs(tabToSkip?: any, skipPinned?: boolean): void;
        /**
         * Internal help to remove all tabs, but the skipTab. If the skipPinned is send as true, skip also all the pinned tabs.
         *
         * @param skipTab: tab to skip
         * @param skipPinned: flag indicating if the pinned tabs should also be skipped (not removed)
         */
        private _CloseAllTabs;
        /**
         * Performs a full refresh of a tab given its identifier
         *
         * @param guid: identifier of the tab to refresh
         */
        RefreshTab(guid: BA.Classes.Guid): void;
        /**
         * Get a Tab by Guid. Returns null if no tab with that guid exists
         *
         * @param guid: identifier of wanted tab
         */
        GetTabByGuid(guid: BA.Classes.Guid): Tab;
        /**
         * Try to find a tab that has a specific record opened (will not work correctly, if that record is opened multiple times)

         * @param oid: identifier of the record searched
         */
        GetTabByRecordOid(oid: BA.Classes.Guid): Tab;
        /**
         * Get a Tab by its reference. Returns null if no tab with that guid exists
         *
         * @param tabReference: reference of the tab searched
         */
        GetTabByReference(tabReference: string): Tab;
        /**
         * Helper to set the IFrame Height to 100%
         */
        FixHeight(): void;
        /**
         * Will hide a Notification based on its class name
         * @param className Class name of the notification
        */
        HideNotificationWithClassName(className: string): void;
        /**
         * Shows a blocker element in the ui
         */
        ShowInputBlocker(): void;
        /**
         * Hides the blocker element in the ui
         */
        HideInputBlocker(): void;
        /**
         * Shows a blocker element in the ui
         */
        ShowAIInputBlocker(): void;
        /**
         * Hides the blocker element in the ui
         */
        HideAIInputBlocker(): void;
        /**
         * Checks if the input blocker is shown. I.e. if the user cannot interact. True means the input is blocked and there should be no interaction with the ui
         */
        IsInputBlocked(): boolean;
        /**
         * Checks if a specific tab can be closed
         *
         * @param guid: Identifier of the tab to check
         */
        GetTabCanClose(guid: string): boolean;
        /**
         * Activates a specific tab given its identifier
         *
         * @param guid: identifier of the tab to activate
         */
        ActivateTabByGuid(guid: string): void;
        /**
         * Closes a specific tab given its identifier
         *
         * @param guid: identifier of the tab to close
         */
        CloseTabByGuid(guid: string, autoUpdateTabBar?: boolean): void;
        CloseActiveTab(autoUpdateTabBar?: boolean): void;
        /**
         * Updates the title of a specific tab given its identifier
         *
         * @param title: new title for the tab
         * @param guid: identifier of the tab
         */
        SetTabTitle(title: string, guid: string): void;
        /**
         * Updates the icon of a specific tab given its identifier
         *
         * @param icon: new icon for the tab
         * @param guid: identifier of the tab
         */
        SetTabIcon(icon: string, guid: string): void;
        /**
         * Returns the title of a specific tab given its identifier
         *
         * @param guid: identifier of the tab
         */
        GetTabTitle(guid: string): string;
        /**
         * Updates the url of a specific tab given its identifier
         *
         * @param url: new url for the tab
         * @param guid: identifier of the tab
         * @param isReplace: if it is not a proper update but a replace, in which case the history entry in the browser will also be replaced
         */
        SetTabUrl(url: string, guid: string, isReplace?: boolean): void;
        /**
         * Updates the reference of a specific tab given its identifier
         *
         * @param ref: new reference for the tab
         * @param guid: identifier of the tab
         */
        SetTabReference(ref: string, guid: string): void;
        AttachToActivatedEvent(guid: string, activateFunc: Function): void;
        RemoveEvents(guid: string): void;
        AttachToActivatedEventOnce(guid: string, activateFunc: Function): void;
        /**
         * If corresponds, executes the autorefresh of a tab given its identifier
         *
         * @param guid: identifier of the tab
         */
        ExecuteAutoRefresh(guid: string): void;
        /**
         * Sets a specific tab's dirty property, given its identifier
         *
         * @param dirty: if the tab should be marked as dirty or cleared
         * @param guid: identifier of the tab
         */
        SetTabDirty(dirty: boolean, guid: string): void;
        /**
         * Checks if a specific tab is dirty, given its identifier
         *
         * @param guid: identifier of the tab
         */
        GetTabDirty(guid: string): boolean;
        /**
         * Checks if a tab is pinned
         *
         * @param guid: guid of the tab to check
         */
        IsTabPinned(guid: string): boolean;
        /**
         * Checks if the active tab is pinned
         */
        IsActiveTabPinned(): boolean;
        /**
         * Sets the autorefresh of the tab from where a tab was opened as true or false
         *
         * @param autoRefresh: True if the tab should be autorefreshed on activate. False otherwise
         * @param guid: identifier of the tab
         */
        SetParentTabAutorefresh(autoRefresh: boolean, guid: string): void;
        /**
         * Sets the autorefresh of a tab as true or false
         *
         * @param autoRefresh: True if the tab should be autorefreshed on activate. False otherwise
         * @param guid: identifier of the tab
         */
        SetTabAutorefresh(autoRefresh: boolean, guid: string): void;
        /**
         * Sets if the autorefresh of a tab should be forced or not, even if the tab would not normally auto refresh.
         *
         * @param autoRefresh: True if the tab should be autorefreshed on activate. False otherwise
         * @param guid: identifier of the tab
         */
        SetForcedTabAutorefresh(forcedAutoRefresh: boolean, guid: string): void;
        /**
         * Checks if a tab is marked as autorefresh given its identifier
         *
         * @param guid: identifier of the tab
         */
        GetTabAutorefresh(guid: string): boolean;
        /**
         * Checks if the autorefresh of a tab should be forced or not, even if the tab would not normally auto refresh
         *
         * @param guid: identifier of the tab
         */
        GetForcedTabAutorefresh(guid: string): boolean;
        /**
         * Marks a specific tab as loaded given its identifier
         *
         * @param guid: identifier of the tab
         */
        SetTabAsLoaded(guid: string): void;
        /**
         * Marks a specific tab as pinned given its identifier
         *
         * @param guid: identifier of the tab
         */
        PinTab(guid: string): void;
        /**
         * Marks a specific tab as not pinned given its identifier
         *
         * @param guid: identifier of the tab
         */
        UnpinTab(guid: string): void;
        /**
         * Sets the record oid for the a specific tab
         *
         * @param oid: record id to set
         * @param guid: identifier of the tab
         */
        SetTabDocumentOid(oid: string, guid: string): void;
        /**
         * Updates parts of a given document given the document id and the ids of the parts to update
         *
         * @param oid: id of the document that needs to have parts updated
         * @param keys: keys of the parts to be updated
         */
        UpdateTabDocumentParts(oid: string, keys: string[]): void;
        /**
         * Checks if a tab with the given tab process id (a.k.a document id) exists
         *
         * @param oid: searched document id
         */
        CheckIfTabExists(oid: string): boolean;
        /**
         * Sets a tab, given its identifier, to be closed after loaded
         *
         * @param guid: identifier of the tab
         */
        SetCloseTabAfterLoad(guid: string): void;
        /**
         * Returns a list of the opened tabs in a json string
         */
        GetTabsAsJSON(): string;
        /**
         * Creates the tabs from the tabs list given as parameters
         *
         * @param tabs: representation of the tabs to create
         */
        AddTabs(tabs: any): void;
        /**
         * Pushes the current tabs to the server
         */
        SaveTabs(): void;
        /**
         * Helper to activate Tab on Browser History change
         */
        WindowStateChange(): void;
        /**
         * Add or update the current Tab to the Browser History
         *
         * @param forceReplace: if the history state should replace the latest one
         */
        SetHistory(forceReplace?: boolean): void;
        /**
         * Pooling time for inter window communication
         */
        PoolInterWindow(): void;
        /**
         * Pooling time for inter window communication
         */
        CheckAlive(): void;
        private IsUrlValid;
        /**
         * Shows the loading animation. If marked as delayed, it waits few milliseconds before showing the animation
         *
         * @param delayed: if the animation should be delayed. Default: true
         */
        ShowWaitAnimation(delayed?: boolean): void;
        /**
         * Hides the loading animation through the controller.ts instance.
         */
        HideWaitAnimation(): void;
        /**
         * Checks if there is a loading animation visible or waiting to be shown
         */
        IsWaitAnimationVisibleOrLoading(): boolean;
    }
    enum ControllerState {
        Undefined = 0,
        Instantiated = 1,
        Busy = 2,
        Idle = 3
    }
}
