﻿using BA.API;
using BA.Core.Enums;
using BA.Core.WorkManager;
using $safeprojectname$.DB;
using $safeprojectname$.Enums.Extensions;
using DevExpress.Xpo;
using System;
using System.Linq;

namespace $safeprojectname$.Worker
{
    /// <summary>
    /// Die Verarbeitung der Einträge der temporären Tabelle wird von dieser Basisklasse automatisch sicher gestellt. 
    /// Daher ist die Komplexität viel geringer.
    /// </summary>
    public class AddServiceMassWorker : TemporaryRecordMassWorkItem<OrmEngine>
    {
        public string Remark { get; set; }
        public DateTime Date { get; set; }
        public int MissingRights { get; set; }

        public AddServiceMassWorker() : base()
        {
            // Der Prozess wird um eine Minute verzögert gestartet.
            ScheduledStartTime = DateTime.UtcNow.AddMinutes(1);

            // Der Prozess kann von dem Benutzer in der UI unterbrochen werden.
            IsCancellable = true;

            // Beschreibende Informationen
            LoggingProcess = EnumLogProcessesExtension.AddService;
            Caption = "281e4152-6d03-40b4-9971-10728e673a00";
            Title = "281e4152-6d03-40b4-9971-10728e673a00";

            // Die ChunkSize gibt an in welchen Paketen die zu verarbeitenden Datensätze geladen werden
            // Default ist 200
            ChunkSize = 500;
        }

        protected override void BeforeProcessing(bool resume)
        {
            if (NumberOfStarts == 1)
            {
                // Erster Start des Prozesses
                Logger.AddInfo("61BCA157-96A6-436B-8719-0521B98462B9");
                Logger.AddInfo("03A9AE68-226F-4F19-91CD-47FA4370230B", MaxProgress);

                // Exception zum Testen des Verhaltens
                throw new Exception(Logger.Translate("160C2DB4-E013-4569-8AB1-A37BDF776707"));
            }
            else
            {
                // Neustart des Prozesses
                Logger.AddInfo("227B79DD-31F2-4F6D-A255-94EB8ACA0E89");
                Logger.AddInfo("E3D32867-83EB-4514-ABEC-C7E596AE192B", CurrentProgress, MaxProgress);
            }
        }

        /// <summary>
        /// Verarbeitung eines Datensatzes
        /// </summary>
        /// <param name="engine"></param>
        protected override void ProcessSingleOrm(OrmEngine engine)
        {
            Logger.AddInfo("1CBA5EFA-7842-4764-ABB8-05E2FAB5BDE3", engine.Name);

            // Exception zum Testen des Verhaltens
            if (CurrentProgress == 2)
                throw new Exception(Logger.Translate("22A14FA7-461E-4813-95A6-75D11EB85B87"));

            // Abprüfung der Rechte
            if (engine.IsAllowed(EnumTableOperations.Edit))
            {
                // Verarbeitung des aktuellen Datensatzes
                OrmSubEngineServices service = engine.Services.AddNewObject();
                service.SortOrder = engine.Services.Count() - 1;
                service.ServiceDate = Date;
                service.Remark = Remark;
            }
            else
            {
                Logger.AddWarning("0DD299DF-C558-44FD-86EC-B3B2ACD47530", engine.Name);
                MissingRights++;
            }
        }

        protected override void AfterProcessing(Exception exception)
        {
            if (MissingRights == 0)
            {
                Api.ClientCommunication.CreateSuccess(Api.User.CurrentUserGuid(), "dd1ceadc-6808-4583-b6b7-dda73188b5a8".Translate(CurrentProgress));
                Logger.AddInfo("dd1ceadc-6808-4583-b6b7-dda73188b5a8", CurrentProgress);
            }
            else
            {
                Api.ClientCommunication.CreateError(Api.User.CurrentUserGuid(), "e3f652e7-8019-4444-9e6c-f3c30528e862".Translate(CurrentProgress - MissingRights, MissingRights));
                Logger.AddInfo("e3f652e7-8019-4444-9e6c-f3c30528e862", CurrentProgress - MissingRights, MissingRights);
            }
        }
    }
}